
// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014-2016 Shandong Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//  __      __      __
//  \ \    /  \    / /   [File name   ] spi.v
//   \ \  / /\ \  / /    [Description ] spi verilog top
//    \ \/ /  \ \/ /     [Timestamp   ] Tue Aug 4 15:30:30 2015
//     \  /    \  /      [version     ] 1.0
//      \/      \/
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |08/07/15  |Initial version
// ===========Oooo==========================================Oooo========


`timescale 1ns/1ps

////////////////////////////////////////////////////////////////////////////
// Width of S_AXI data bus. The slave accepts write data and issues read data
// of width C_S_AXI_DATA_WIDTH
`define C_S_AXI_DATA_WIDTH 32
`define C_S_AXI_ADDR_WIDTH 32
module spi #(
   parameter  SHIFT_DIRECTION = 0,
   parameter  CLOCK_PHASE     = 0,
   parameter  CLOCK_POLARITY  = 0,
   parameter  CLOCK_SEL       = 1,		//  SCLK = CLK_I / (2 * (CLOCK_SEL + 1)) = CLK_I / 4
   parameter  MASTER          = 1,
   parameter  SLAVE_NUMBER    = 1,		//  maximum value = 8 for current design
   parameter  DATA_LENGTH     = 32,		//  changed from 32 to 8
   parameter  DELAY_TIME      = 2,
   parameter  CLKCNT_WIDTH    = 5,
   parameter  INTERVAL_LENGTH = 2)
(
  //slave port
   input  wire                             S_AXI_ACLK,      // AXI clock signal
   input  wire                             S_AXI_ARESETN,   // AXI active low reset signal
   input  wire [`C_S_AXI_ADDR_WIDTH - 1:0] S_AXI_AWADDR,
   input  wire                             S_AXI_AWVALID,
   output wire                             S_AXI_AWREADY,
   input  wire [`C_S_AXI_DATA_WIDTH-1:0]   S_AXI_WDATA,
   input  wire [`C_S_AXI_DATA_WIDTH/8-1:0] S_AXI_WSTRB,
   input  wire                             S_AXI_WVALID,
   output wire                             S_AXI_WREADY,
   output wire [1:0]                       S_AXI_BRESP,
   output wire                             S_AXI_BVALID,
   input  wire                             S_AXI_BREADY,
   input  wire [`C_S_AXI_ADDR_WIDTH - 1:0] S_AXI_ARADDR,
   input  wire                             S_AXI_ARVALID,
   output wire                             S_AXI_ARREADY,
   output wire [`C_S_AXI_DATA_WIDTH-1:0]   S_AXI_RDATA,
   output wire [1:0]                       S_AXI_RRESP,
   output wire                             S_AXI_RVALID,
   input  wire                             S_AXI_RREADY,
    
   output wire                             SPI_INT_O,
 //spi interface
   input  wire                             MISO_MASTER,
   output reg                              MOSI_MASTER,
   output reg  [SLAVE_NUMBER-1:0]          SS_N_MASTER,
   output reg                              SCLK_MASTER,
   output reg                              MISO_SLAVE,
   input  wire                             MOSI_SLAVE,
   input  wire                             SS_N_SLAVE,
   input  wire                             SCLK_SLAVE
);
   
////////////////////////////////////////////////////////////////////////////
// function called clogb2 that returns an integer which has the
// value of the ceiling of the log base 2.
 function integer clogb2 (input integer bd);
 integer bit_depth;
   begin
   bit_depth = bd;
   for(clogb2=0; bit_depth>0; clogb2=clogb2+1)
     bit_depth = bit_depth >> 1;
   end
 endfunction

 localparam integer ADDR_LSB = clogb2(`C_S_AXI_DATA_WIDTH/8)-1;
 localparam integer ADDR_MSB = `C_S_AXI_ADDR_WIDTH;

////////////////////////////////////////////////////////////////////////////
// AXI4 Lite internal signals
   reg [1 :0]                     axi_rresp;    // read response
   reg [1 :0]                     axi_bresp;    // write response
   reg                            axi_awready;  // write address acceptance
   reg                            axi_wready;   // write data acceptance
   reg                            axi_bvalid;   // write response valid
   reg                            axi_rvalid;   // read data valid
   reg [ADDR_MSB-1:0]             axi_awaddr;   // write address
   reg [ADDR_MSB-1:0]             axi_araddr;   // read address valid
   reg [`C_S_AXI_DATA_WIDTH-1:0]  axi_rdata;    // read data
   reg                            axi_arready;  // read address acceptance

   wire                           slv_reg_rden; // Slave register read enable
   wire                           slv_reg_wren; // Slave register write enable
   reg [`C_S_AXI_DATA_WIDTH-1:0]  reg_data_out; // register read data
   integer                        byte_index;
 ////////////////////////////////////////////////////////////////////////////
// SPI internal signals 
   
   reg [DATA_LENGTH-1:0]   reg_rxdata;
   reg [DATA_LENGTH-1:0]   reg_txdata;
   reg [DATA_LENGTH-1:0]   rx_shift_data;
   reg [DATA_LENGTH-1:0]   tx_shift_data;
   reg                     rx_latch_flag;
   reg [SLAVE_NUMBER-1:0]  reg_ssmask;
   
   reg [7:0]               reg_control;			//8 bit width - changed from 11 bits
   wire                    reg_iroe;
   wire                    reg_itoe;
   wire                    reg_itrdy;
   wire                    reg_irrdy;
   wire                    reg_ie;
   wire                    reg_sso;

   wire [7:0 ]             reg_status;			//8 bit width - changed from 9 bits
   reg                     reg_toe;
   reg                     reg_roe;
   reg                     reg_trdy;
   reg                     reg_rrdy;
   reg                     reg_tmt;
   wire                    reg_e;

   parameter               UDLY           = 1;
   parameter               ST_IDLE        = 3'b000;
   parameter               ST_LOAD        = 3'b001;
   parameter               ST_WAIT        = 3'b010;
   parameter               ST_TRANS       = 3'b011;
   parameter               ST_TURNAROUND  = 3'b100;
   parameter               ST_INTERVAL    = 3'b101;


////////////////////////////////////////////////////////////////////////////
//Write Address Ready (AWREADY)
 assign S_AXI_AWREADY = axi_awready;  //Write Address Ready (AWREADY)
 assign S_AXI_WREADY  = axi_wready;  //Write Data Ready(WREADY)
 assign S_AXI_BRESP   = axi_bresp;  //Write Response (BResp)and response valid (BVALID)
 assign S_AXI_BVALID  = axi_bvalid;
 assign S_AXI_ARREADY = axi_arready;  //Read Address Ready(AREADY)
 assign S_AXI_RDATA   = axi_rdata;  //Read and Read Data (RDATA), Read Valid (RVALID) and Response (RRESP)
 assign S_AXI_RVALID  = axi_rvalid;
 assign S_AXI_RRESP   = axi_rresp;


////////////////////////////////////////////////////////////////////////////
// Implement axi_awready generation
  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_awready <= 1'b0;
      end
    else
      begin
        if (~axi_awready && S_AXI_AWVALID && S_AXI_WVALID)
          begin
            axi_awready <= 1'b1;
          end
        else
          begin
            axi_awready <= 1'b0;
          end
      end
  end

////////////////////////////////////////////////////////////////////////////
// Implement axi_awaddr latching
  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_awaddr <= 0;
      end
    else
      begin
        if (~axi_awready && S_AXI_AWVALID && S_AXI_WVALID)
          begin
           axi_awaddr <= S_AXI_AWADDR;
          end
      end
  end

////////////////////////////////////////////////////////////////////////////
// Implement axi_wready generation
  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_wready <= 1'b0;
      end
    else
      begin
        if (~axi_wready && S_AXI_WVALID && S_AXI_AWVALID)
          begin
            axi_wready <= 1'b1;
          end
        else
          begin
            axi_wready <= 1'b0;
          end
      end
  end

////////////////////////////////////////////////////////////////////////////
// Implement memory mapped register select and write logic generation
  assign slv_reg_wren = axi_wready && S_AXI_WVALID && axi_awready && S_AXI_AWVALID;

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        reg_txdata  <= 'h0;
        reg_control <= 'h0;
        reg_ssmask  <= 'h0;
      end
    else begin
      if (slv_reg_wren)
        begin
          case ( axi_awaddr[ADDR_MSB-1:ADDR_LSB] )
         
            3'h1 : //Transmit Data Register(0x04)
                for ( byte_index = 0; byte_index <= (`C_S_AXI_DATA_WIDTH/8)-1; byte_index = byte_index+1 )
                if ( S_AXI_WSTRB[byte_index] == 1 ) begin
                    reg_txdata[(byte_index*8) +: 8] <= S_AXI_WDATA[(byte_index*8) +: 8];
                end
            3'h3 :
               for ( byte_index = 0; byte_index <= (`C_S_AXI_DATA_WIDTH/8)-1; byte_index = byte_index+1 )
               if ( S_AXI_WSTRB[byte_index] == 1 ) begin
                    reg_control[(byte_index*8) +: 8] <= S_AXI_WDATA[(byte_index*8) +: 8];
                 end
            3'h4 :
             if(SLAVE_NUMBER <= 8)
              reg_ssmask[SLAVE_NUMBER-1:0]<= S_AXI_WDATA[SLAVE_NUMBER-1:0];
             else
             begin
               for ( byte_index = 0; byte_index <= (`C_S_AXI_DATA_WIDTH/8)-1; byte_index = byte_index+1 )
               if ( S_AXI_WSTRB[byte_index] == 1 ) begin
                    reg_ssmask[(byte_index*8) +: 8] <= S_AXI_WDATA[(byte_index*8) +: 8];
               end
             end

            default : begin
                        reg_txdata  <= reg_txdata;
                        reg_control <= reg_control;
                        reg_ssmask  <= reg_ssmask;
                      end
          endcase
        end
    end
  end

////////////////////////////////////////////////////////////////////////////
// Implement write response logic generation

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_bvalid  <= 0;
        axi_bresp   <= 2'b0;
      end
    else
      begin
        if (axi_awready && S_AXI_AWVALID && ~axi_bvalid && axi_wready && S_AXI_WVALID)
          begin
            // indicates a valid write response is available
            axi_bvalid <= 1'b1;
            axi_bresp  <= 2'b0; // 'OKAY' response
          end                   // work error responses in future
        else
          begin
            if (S_AXI_BREADY && axi_bvalid)
              //check if bready is asserted while bvalid is high)
              //(there is a possibility that bready is always asserted high)
              begin
                axi_bvalid <= 1'b0;
              end
          end
      end
  end


////////////////////////////////////////////////////////////////////////////
// Implement axi_arready generation

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_arready <= 1'b0;
        axi_araddr  <= {ADDR_MSB{1'b0}};
      end
    else
      begin
        if (~axi_arready && S_AXI_ARVALID)
          begin
            // indicates that the slave has acceped the valid read address
            axi_arready <= 1'b1;
            axi_araddr  <= S_AXI_ARADDR;
          end
        else
          begin
            axi_arready <= 1'b0;
          end
      end
  end

////////////////////////////////////////////////////////////////////////////
// Implement memory mapped register select and read logic generation
  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_rvalid <= 0;
        axi_rresp  <= 0;
      end
    else
      begin
        if (axi_arready && S_AXI_ARVALID && ~axi_rvalid)
          begin
            // Valid read data is available at the read data bus
            axi_rvalid <= 1'b1;
            axi_rresp  <= 2'b0; // 'OKAY' response
          end
        else if (axi_rvalid && S_AXI_RREADY)
          begin
            // Read data is accepted by the master
            axi_rvalid <= 1'b0;
          end
      end
  end


////////////////////////////////////////////////////////////////////////////
// Slave register read enable is asserted when valid address is available
// and the slave is ready to accept the read address.
  assign slv_reg_rden = axi_arready & S_AXI_ARVALID & ~axi_rvalid;

  always @( reg_rxdata, reg_txdata, reg_status, reg_control,reg_ssmask, S_AXI_ARESETN, slv_reg_rden, axi_araddr)
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        reg_data_out <= {`C_S_AXI_DATA_WIDTH{1'b0}};
      end
    else
      begin
        // Read address mux
        case ( axi_araddr[ADDR_MSB-1:ADDR_LSB] )
          3'h0   : reg_data_out <= reg_rxdata;
          3'h1   : reg_data_out <= reg_txdata;
          3'h2   : reg_data_out <= reg_status;
          3'h3   : reg_data_out <= reg_control;
          3'h4   : reg_data_out <= reg_ssmask;
          default : reg_data_out <= {`C_S_AXI_DATA_WIDTH{1'b0}};
        endcase
        //end
      end
  end

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_rdata  <= 0;
      end
    else
      begin
        ////////////////////////////////////////////////////////////////////////////
        // When there is a valid read address (S_AXI_ARVALID) with
        // acceptance of read address by the slave (axi_arready),
        // output the read dada
        if (axi_arready && S_AXI_ARVALID && ~axi_rvalid)
          begin
            axi_rdata <= reg_data_out;     // register read data
          end
      end
  end


//Receive Data Register(0x00)
   always @(posedge S_AXI_ACLK)
     if(S_AXI_ARESETN == 1'b0)
       reg_rxdata            <= #UDLY 'h0;
     else if (rx_latch_flag)
       reg_rxdata            <= #UDLY rx_shift_data;


//Status Register(0x08)		// reduced status register length to 8 bits - changed padding at end from 3 bits
   assign      reg_e       = reg_toe | reg_roe;
   assign      reg_status  = {reg_e,reg_rrdy,reg_trdy,reg_tmt,reg_toe,reg_roe,2'b00};

   assign  SPI_INT_O = (reg_ie & ((reg_iroe & reg_roe) | (reg_itoe & reg_toe))) |
                       (reg_itrdy & reg_trdy) |
                       (reg_irrdy & reg_rrdy);

generate
if (MASTER == 1) begin
   //Master Only registers
   reg [CLKCNT_WIDTH-1:0]  clock_cnt;
   reg  [5:0]              data_cnt;
   reg                     pending_data;
   reg [2:0]               c_status;
   reg [2:0]               n_status;

					// reduced control register length to 8 bits - deleted 3 bits padding at end
   //assign  reg_control     = {reg_sso,1'b0,reg_ie,reg_irrdy,reg_itrdy,1'b0,reg_itoe,reg_iroe};
   assign   reg_iroe    =  reg_control[0];			//  was bit 3
   assign   reg_itoe    =  reg_control[1];			//  was bit 4
   assign   reg_itrdy   =  reg_control[3];			//  was bit 6
   assign   reg_irrdy   =  reg_control[4];			//  was bit 7
   assign   reg_ie      =  reg_control[5];			//  was bit 8
   assign   reg_sso     =  reg_control[7];    		//  was bit 10 
   
   always @(posedge S_AXI_ACLK)
     if(S_AXI_ARESETN == 1'b0)
       pending_data               <= #UDLY 1'b0;
     else if (slv_reg_wren&&(axi_awaddr[ADDR_MSB-1:ADDR_LSB]==3'h1))
       pending_data               <= #UDLY 1'b1;
     else if (c_status == ST_LOAD)
       pending_data               <= #UDLY 1'b0;


   always @(posedge S_AXI_ACLK) begin
     if (S_AXI_ARESETN == 1'b0)
        SS_N_MASTER <= {SLAVE_NUMBER{1'b1}};
     else if (reg_sso)
        SS_N_MASTER <= (~reg_ssmask);
     else if ((c_status == ST_TRANS) || (c_status == ST_WAIT) || (c_status == ST_TURNAROUND))
        SS_N_MASTER <= (~reg_ssmask);
     else
        SS_N_MASTER <= {SLAVE_NUMBER{1'b1}};
     end

   // Generate SCLK
   always @(posedge S_AXI_ACLK)
     if (S_AXI_ARESETN == 1'b0)
        SCLK_MASTER    <= #UDLY CLOCK_POLARITY;
     else if ((c_status == ST_TRANS)  && (clock_cnt == CLOCK_SEL))
        SCLK_MASTER    <= #UDLY ~SCLK_MASTER;

   always @(posedge S_AXI_ACLK)
     if (S_AXI_ARESETN == 1'b0)
        rx_shift_data       <= #UDLY 'h0;
     else if ((clock_cnt == CLOCK_SEL) && (!(CLOCK_PHASE ^ CLOCK_POLARITY ^ SCLK_MASTER)) && (c_status == ST_TRANS))  begin
        if (SHIFT_DIRECTION)
           rx_shift_data    <= #UDLY {MISO_MASTER,rx_shift_data[DATA_LENGTH-1:1]};
        else
           rx_shift_data    <= #UDLY {rx_shift_data,MISO_MASTER};
     end

   // control when the data is received over
   always @(posedge S_AXI_ACLK) begin
      if (S_AXI_ARESETN == 1'b0)
         rx_latch_flag <= 1'b0;
      else if ((c_status == ST_TRANS) && (n_status !== ST_TRANS))
         rx_latch_flag <= 1'b1;
      else if (rx_latch_flag)
         rx_latch_flag <= 1'b0;
      end

   always @(posedge S_AXI_ACLK)
      if (S_AXI_ARESETN == 1'b0)
         clock_cnt <= 0;
      else if (clock_cnt == CLOCK_SEL || (n_status == ST_IDLE))
         clock_cnt <= 0;
      else
         clock_cnt <= clock_cnt + 1;

   always @(posedge S_AXI_ACLK) begin
      if (S_AXI_ARESETN == 1'b0)
         c_status <= ST_IDLE;
      else
         c_status <= n_status;
      end

   wire [5:0] ACTUAL_MAX = (CLOCK_POLARITY == CLOCK_PHASE) ?
                          DATA_LENGTH - 1 :
                          DATA_LENGTH;

     always @(*)
       begin
       if (S_AXI_ARESETN == 1'b0)
          n_status <= ST_IDLE;
       else
       case(c_status)
       ST_IDLE:   if (pending_data)
                       n_status <= ST_LOAD;
                  else
                       n_status <= ST_IDLE;
       ST_LOAD:  begin
                     if (DELAY_TIME == 0)
                         n_status <= ST_TRANS;
                     else
                         n_status <= ST_WAIT;
                  end
       ST_WAIT:   begin
                     if ((clock_cnt == CLOCK_SEL) && (data_cnt == DELAY_TIME - 1))
                         n_status <= ST_TRANS;
                     else
                         n_status <= ST_WAIT;
                  end
       ST_TRANS:  begin
                     if ((clock_cnt == CLOCK_SEL) &&
                         (data_cnt == ACTUAL_MAX) &&
                         (SCLK_MASTER != CLOCK_POLARITY))
                         n_status <= ST_TURNAROUND;
                     else
                         n_status <= ST_TRANS;
                  end
       ST_TURNAROUND:  begin
                        if (clock_cnt == CLOCK_SEL)
                          if (INTERVAL_LENGTH)
                             n_status <= ST_INTERVAL;
                          else
                             n_status <= ST_IDLE;
                       end
       ST_INTERVAL:  begin
                       if ((clock_cnt == CLOCK_SEL) && (data_cnt == INTERVAL_LENGTH))
                          n_status <= ST_IDLE;
                       else
                          n_status <= ST_INTERVAL;
                       end
       default:     n_status <= ST_IDLE;
       endcase
       end

   always @(posedge S_AXI_ACLK)  begin
      if (S_AXI_ARESETN == 1'b0)
         data_cnt <= 0;
      else if ((c_status == ST_WAIT)   && (clock_cnt == CLOCK_SEL) &&  (data_cnt == DELAY_TIME - 1))
         data_cnt <= 0;
      else if ((c_status == ST_TRANS)  && (clock_cnt == CLOCK_SEL) && (data_cnt == ACTUAL_MAX) && (CLOCK_POLARITY != SCLK_MASTER))
         data_cnt <= 0;
      else if ((c_status == ST_INTERVAL) && (clock_cnt == CLOCK_SEL) && (data_cnt == INTERVAL_LENGTH))
         data_cnt <= 0;
      else if (((c_status == ST_WAIT)   && (clock_cnt == CLOCK_SEL)) ||
               ((c_status == ST_TRANS)  && (clock_cnt == CLOCK_SEL)  && (CLOCK_PHASE != SCLK_MASTER))||
               ((c_status == ST_INTERVAL) && (clock_cnt == CLOCK_SEL)))
         data_cnt <= data_cnt + 1;
      end

   reg wait_one_tick_done;
   always @(posedge S_AXI_ACLK) begin
      if (S_AXI_ARESETN == 1'b0)
          wait_one_tick_done <= 1'b0;
      else if (CLOCK_PHASE && (((c_status == ST_LOAD) && (n_status == ST_TRANS)) ||
                               ((c_status == ST_WAIT) && (n_status == ST_TRANS))))
          wait_one_tick_done <= 1'b1;
      else if ((c_status == ST_TRANS) && (clock_cnt == CLOCK_SEL) && (CLOCK_PHASE ^ CLOCK_POLARITY ^ SCLK_MASTER) )
          wait_one_tick_done <= 1'b0;
      end

    always @(posedge S_AXI_ACLK) begin
      if (S_AXI_ARESETN == 1'b0) begin
         MOSI_MASTER <= 0;
         tx_shift_data <= 0;  end
      else if (((c_status == ST_LOAD) && (n_status == ST_TRANS)) ||
               ((c_status == ST_WAIT) && (n_status == ST_TRANS))) begin
             MOSI_MASTER   <= SHIFT_DIRECTION ? reg_txdata[0] :
                                                reg_txdata[DATA_LENGTH-1];
             tx_shift_data <= SHIFT_DIRECTION ? {1'b0, reg_txdata[DATA_LENGTH-1:1]} :
                                                {reg_txdata, 1'b0};
             end
      else if ((!wait_one_tick_done) && (c_status == ST_TRANS) && (clock_cnt == CLOCK_SEL) && (CLOCK_PHASE ^ CLOCK_POLARITY ^ SCLK_MASTER) )
            begin
              MOSI_MASTER   <= SHIFT_DIRECTION ? tx_shift_data[0] :
                                                 tx_shift_data[DATA_LENGTH-1];
              tx_shift_data <= SHIFT_DIRECTION ? {1'b0, tx_shift_data[DATA_LENGTH-1:1]} :
                                                 {tx_shift_data, 1'b0};
            end
      end

   always @(posedge S_AXI_ACLK)
      if (S_AXI_ARESETN == 1'b0)
         reg_trdy <= 1;
      else if ((c_status != ST_TRANS) && (n_status == ST_TRANS))
         reg_trdy <= 1;
      else if (slv_reg_wren && (axi_awaddr[ADDR_MSB-1:ADDR_LSB]==3'h1) )
         reg_trdy <= 0;

   always @(posedge S_AXI_ACLK)
      if (S_AXI_ARESETN == 1'b0)
         reg_toe <= 0;
      else if (!reg_trdy && slv_reg_wren && (axi_awaddr[ADDR_MSB-1:ADDR_LSB]==3'h1) )
         reg_toe <= 1'b1;
      else if (slv_reg_wren && (axi_awaddr[ADDR_MSB-1:ADDR_LSB]==3'h2) && S_AXI_WDATA[3])
         reg_toe <= 1'b0;

   always @(posedge S_AXI_ACLK)
      if (S_AXI_ARESETN == 1'b0)  begin
         reg_rrdy <= 1'b0;
         reg_roe  <= 1'b0;  end
      else if ((c_status == ST_TURNAROUND) && (clock_cnt == CLOCK_SEL)) begin
         if (reg_rrdy)
            reg_roe <= 1'b1;
         else  begin
            reg_rrdy <= 1'b1;
            end
         end
      else if (slv_reg_wren && (axi_awaddr[ADDR_MSB-1:ADDR_LSB]==3'h2) && S_AXI_WDATA[2])  begin
         reg_roe  <= 1'b0; end
      else if (slv_reg_rden && (axi_araddr[ADDR_MSB-1:ADDR_LSB]==3'h0)) begin
         reg_rrdy <= 1'b0; end

   always @(posedge S_AXI_ACLK)
      if (S_AXI_ARESETN == 1'b0)
         reg_tmt <= 1'b1;
      else if ((c_status != ST_IDLE) || pending_data)
         reg_tmt <= 1'b0;
      else
         reg_tmt <= 1'b1;

  end
  endgenerate
  //End of Master Section

  //Slave Mode
generate

if (MASTER == 0)
begin
   //Slave Only registers/wires
   reg                     tx_done;
   reg                     rx_done;
   reg                     rx_done_flip1;
   reg                     rx_done_flip2;
   reg                     tx_done_flip1;
   reg                     tx_done_flip2;
   reg  [5:0]              rx_data_cnt;
   reg  [5:0]              tx_data_cnt;

					// reduced control register length to 8 bits - deleted 3 bits padding at end
  // assign  reg_control     = {1'b0,   1'b0,reg_ie,reg_irrdy,reg_itrdy,1'b0,reg_itoe,reg_iroe};
   assign   reg_iroe    =  reg_control[0];			//  was bit 3
   assign   reg_itoe    =  reg_control[1];			//  was bit 4
   assign   reg_itrdy   =  reg_control[3];			//  was bit 6
   assign   reg_irrdy   =  reg_control[4];			//  was bit 7
   assign   reg_ie      =  reg_control[5];			//  was bit 8
  

   //For Rx data, sample at posedge when CLOCK_PHASE ^~ CLOCK_POLARITY is 1
if (!(CLOCK_PHASE ^ CLOCK_POLARITY))
begin
     always @(posedge SCLK_SLAVE or negedge S_AXI_ARESETN)
       if (S_AXI_ARESETN == 1'b0)
         rx_shift_data     <= #UDLY 'h0;
       else if (!SS_N_SLAVE)
           if (SHIFT_DIRECTION)
              rx_shift_data   <= #UDLY {MOSI_SLAVE,rx_shift_data[DATA_LENGTH-1:1]};
           else
              rx_shift_data   <= #UDLY {rx_shift_data,MOSI_SLAVE};

     always @(posedge SCLK_SLAVE or negedge S_AXI_ARESETN)
       if (S_AXI_ARESETN == 1'b0)
          rx_data_cnt     <= #UDLY 'h0;
       else if (rx_data_cnt == DATA_LENGTH - 1)
          rx_data_cnt     <= #UDLY 'h0;
       else if (!SS_N_SLAVE)
          rx_data_cnt     <= #UDLY rx_data_cnt + 1;

     always @(posedge SCLK_SLAVE or negedge S_AXI_ARESETN)
       if (S_AXI_ARESETN == 1'b0)
          rx_done         <= #UDLY 1'b0;
       else if (rx_data_cnt == DATA_LENGTH - 1)
          rx_done         <= #UDLY 1'b1;
       else
          rx_done         <= #UDLY 1'b0;

end
else
begin
    //For Rx data, sample at negedge when CLOCK_PHASE ^~ CLOCK_POLARITY is 0
    always @(negedge SCLK_SLAVE or negedge S_AXI_ARESETN)
       if (S_AXI_ARESETN == 1'b0)
         rx_shift_data     <= #UDLY 'h0;
       else if (!SS_N_SLAVE)
           if (SHIFT_DIRECTION)
              rx_shift_data   <= #UDLY {MOSI_SLAVE,rx_shift_data[DATA_LENGTH-1:1]};
           else
              rx_shift_data   <= #UDLY {rx_shift_data,MOSI_SLAVE};

     always @(negedge SCLK_SLAVE or negedge S_AXI_ARESETN)
       if (S_AXI_ARESETN == 1'b0)
          rx_data_cnt     <= #UDLY 'h0;
       else if (rx_data_cnt == DATA_LENGTH - 1)
          rx_data_cnt     <= #UDLY 'h0;
       else if (!SS_N_SLAVE)
          rx_data_cnt     <= #UDLY rx_data_cnt + 1;

     always @(negedge SCLK_SLAVE or negedge S_AXI_ARESETN )
       if (S_AXI_ARESETN == 1'b0)
          rx_done         <= #UDLY 1'b0;
       else if (rx_data_cnt == DATA_LENGTH - 1)
          rx_done         <= #UDLY 1'b1;
       else
          rx_done         <= #UDLY 1'b0;
end

   always @(posedge S_AXI_ACLK)
     if (S_AXI_ARESETN == 1'b0) begin
       rx_done_flip1                <= #UDLY 1'b0;
       rx_done_flip2                <= #UDLY 1'b0;end
     else begin
       rx_done_flip1                <= #UDLY rx_done;
       rx_done_flip2                <= #UDLY rx_done_flip1;end

   always @(posedge S_AXI_ACLK)
     if (S_AXI_ARESETN == 1'b0)
       rx_latch_flag                <= #UDLY 1'b0;
     else if (rx_done_flip1 && !rx_done_flip2)
       rx_latch_flag                <= #UDLY 1'b1;
     else
       rx_latch_flag                <= #UDLY 1'b0;

   always @(posedge S_AXI_ACLK)
     if (S_AXI_ARESETN == 1'b0)
       reg_rrdy                     <= #UDLY 1'b0;
     else if (rx_done_flip1 && !rx_done_flip2)
       reg_rrdy                     <= #UDLY 1'b1;
     else if ((axi_araddr[ADDR_MSB-1:ADDR_LSB]==3'h0)&& slv_reg_rden)
       reg_rrdy                     <= #UDLY 1'b0;

   always @(posedge S_AXI_ACLK)
     if (S_AXI_ARESETN == 1'b0)
       reg_roe                      <= #UDLY 1'b0;
     else if (rx_done_flip1 && !rx_done_flip2 && reg_rrdy)
       reg_roe                      <= #UDLY 1'b1;
     else if (slv_reg_wren && (axi_awaddr[ADDR_MSB-1:ADDR_LSB]==3'h2) && S_AXI_WDATA[2])
       reg_roe                      <= #UDLY 1'b0;

   //For Tx data, update at negedge when CLOCK_PHASE =0,CLOCK_POLARITY=0;
   //             update at posedge when CLOCK_PHASE =0,CLOCK_POLARITY=1;

if (CLOCK_PHASE ^ CLOCK_POLARITY)
begin
   if(CLOCK_PHASE)
   begin
     always @(posedge SCLK_SLAVE or negedge S_AXI_ARESETN)
       if (S_AXI_ARESETN == 1'b0)
         MISO_SLAVE   <= #UDLY 'h0;
       else
         MISO_SLAVE   <= #UDLY SHIFT_DIRECTION ? reg_txdata[tx_data_cnt] :
                                                 reg_txdata[DATA_LENGTH-tx_data_cnt-1];
  end
  else
  begin
       always @(*)
         MISO_SLAVE   <= #UDLY SHIFT_DIRECTION ? reg_txdata[tx_data_cnt] :
                                                 reg_txdata[DATA_LENGTH-tx_data_cnt-1];
  end

     always @(posedge SCLK_SLAVE or negedge S_AXI_ARESETN)
       if (S_AXI_ARESETN == 1'b0)
          tx_data_cnt     <= #UDLY 'h0;
       else if (tx_data_cnt == DATA_LENGTH - 1)
          tx_data_cnt     <= #UDLY 'h0;
       else if (!SS_N_SLAVE)
          tx_data_cnt     <= #UDLY tx_data_cnt + 1;

     always @(posedge SCLK_SLAVE or negedge S_AXI_ARESETN)
       if (S_AXI_ARESETN == 1'b0)
          tx_done     <= #UDLY 1'b0;
       else if (tx_data_cnt == DATA_LENGTH - 1)
          tx_done     <= #UDLY 1'b1;
       else
          tx_done     <= #UDLY 1'b0;
   //For Tx data, update at negedge when CLOCK_PHASE = 1,CLOCK_POLARITY == 1;
   //             update at posedge when CLOCK_PHASE = 1,CLOCK_POLARITY == 0; 
end
else
begin
   if(CLOCK_PHASE)
   begin
     always @(negedge SCLK_SLAVE or negedge S_AXI_ARESETN )
       if (S_AXI_ARESETN == 1'b0)
         MISO_SLAVE   <= #UDLY 'h0;
       else
         MISO_SLAVE   <= #UDLY SHIFT_DIRECTION ? reg_txdata[tx_data_cnt] :
                                                 reg_txdata[DATA_LENGTH-tx_data_cnt-1];
   end
   else
   begin
       always @(*)
         MISO_SLAVE   <= #UDLY SHIFT_DIRECTION ? reg_txdata[tx_data_cnt] :
                                                 reg_txdata[DATA_LENGTH-tx_data_cnt-1];
   end

     always @(negedge SCLK_SLAVE or negedge S_AXI_ARESETN )
       if (S_AXI_ARESETN == 1'b0)
          tx_data_cnt     <= #UDLY 'h0;
       else if (tx_data_cnt == DATA_LENGTH - 1)
          tx_data_cnt     <= #UDLY 'h0;
       else if (!SS_N_SLAVE)
          tx_data_cnt     <= #UDLY tx_data_cnt + 1;

     always @(negedge SCLK_SLAVE or negedge S_AXI_ARESETN )
       if (S_AXI_ARESETN == 1'b0)
          tx_done     <= #UDLY 1'b0;
       else if (tx_data_cnt == DATA_LENGTH - 1)
          tx_done     <= #UDLY 1'b1;
       else
          tx_done     <= #UDLY 1'b0;
end

   always @(posedge S_AXI_ACLK)
     if (S_AXI_ARESETN == 1'b0) begin
       tx_done_flip1                <= #UDLY 1'b0;
       tx_done_flip2                <= #UDLY 1'b0;end
     else begin
       tx_done_flip1                <= #UDLY tx_done;
       tx_done_flip2                <= #UDLY tx_done_flip1;end

   always @(posedge S_AXI_ACLK)
     if (S_AXI_ARESETN == 1'b0)
        reg_trdy     <= #UDLY 1'b1;
     else if (slv_reg_wren && (axi_awaddr[ADDR_MSB-1:ADDR_LSB]==3'h1))
        reg_trdy     <= #UDLY 1'b0;
     else if (tx_done_flip1 && !tx_done_flip2)
        reg_trdy     <= #UDLY 1'b1;

   always @(posedge S_AXI_ACLK)
     if (S_AXI_ARESETN == 1'b0)
        reg_tmt     <= #UDLY 1'b1;
     else if (slv_reg_wren && (axi_awaddr[ADDR_MSB-1:ADDR_LSB]==3'h1))
        reg_tmt     <= #UDLY 1'b0;
     else if (tx_done_flip1 && !tx_done_flip2)
        reg_tmt     <= #UDLY 1'b1;

   always @(posedge S_AXI_ACLK)
     if(S_AXI_ARESETN == 1'b0)
       reg_toe      <= #UDLY 1'b0;
     else if(!reg_trdy && slv_reg_wren && (axi_awaddr[ADDR_MSB-1:ADDR_LSB]==3'h1))
       reg_toe      <= #UDLY 1'b1;
     else if(slv_reg_wren && (axi_awaddr[ADDR_MSB-1:ADDR_LSB]==3'h2) && S_AXI_WDATA[3])
       reg_toe      <= #UDLY 1'b0;

end
endgenerate

endmodule
