
// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014-2017 Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//
//  __      __      __
//  \ \    /  \    / /   [File name   ] spi_slave.v
//   \ \  / /\ \  / /    [Description ] SPI Slave Model
//    \ \/ /  \ \/ /     [Timestamp   ] Friday July 04 10:00:30 2017
//     \  /    \  /      [version     ] 1.0.0
//      \/      \/       
//
// ===========Oooo==========================================Oooo========
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |08/02/17  |Initial version
// ===========Oooo==========================================Oooo========
`timescale 1ns / 1ps

`include "spi_define.v"

module spi_slave (
    SCLK,
    MOSI,
    MISO,
    SS
);

/********************************************************************
*port and variables
********************************************************************/
    input                       SCLK;
    input                       MOSI;
    input                       SS;
    output reg                  MISO;

    reg [5:0]                   rx_cnt                  =       0;
    reg [5:0]                   tx_cnt                  =       0;
    reg [`DATA_LENGTH-1:0]      mosi_shift_reg          =       0;
    reg [`DATA_LENGTH-1:0]      miso_shift_reg          =       0;


/*********************************************************************
*receive data 
*********************************************************************/
if(!(`CLOCK_POLARITY ^ `CLOCK_PHASE))begin
    always@(posedge SCLK )
        if(SS)
            mosi_shift_reg <= 0;
        else if(!SS && (rx_cnt < `DATA_LENGTH))begin
            if(`SHIFT_DIRECTION)    //LSB -> MSB
                mosi_shift_reg <= {MOSI,mosi_shift_reg[`DATA_LENGTH-1:1]};
            else                    //MSB -> LSB
                mosi_shift_reg <= {mosi_shift_reg[`DATA_LENGTH-2:0],MOSI};
        end
        else
            mosi_shift_reg <= 0;

    always@(posedge SCLK )
        if(SS)
            rx_cnt <= 0;
        else if(rx_cnt == `DATA_LENGTH - 1)
            rx_cnt <= 0;
        else 
            rx_cnt <= rx_cnt + 1;
end
else begin
    always@(negedge SCLK /*or negedge SS*/)
        if(SS)
            mosi_shift_reg <= 0;
        else if(!SS && (rx_cnt < `DATA_LENGTH))begin
            if(`SHIFT_DIRECTION)        //LSB -> MSB
                mosi_shift_reg  <=  {MOSI,mosi_shift_reg[`DATA_LENGTH-1:1]};
            else                        //MSB -> LSB
                mosi_shift_reg  <=  {mosi_shift_reg[`DATA_LENGTH-2:0],MOSI};
        end
        else
            mosi_shift_reg <= 0;
     always@(negedge SCLK /*or negedge SS*/)
        if(SS)
            rx_cnt <= 0;
        else if(rx_cnt == `DATA_LENGTH - 1)
            rx_cnt <= 0;
        else if(!SS)
            rx_cnt <= rx_cnt + 1;
end

/*******************************************************************
*transmit data 
*******************************************************************/
if(`CLOCK_POLARITY ^ `CLOCK_PHASE)begin
    always@(posedge SCLK )
        if(SS)
            tx_cnt <= 0;
        else if(tx_cnt >= `DATA_LENGTH - 1)begin
            miso_shift_reg <= mosi_shift_reg;
            tx_cnt <= 0;
        end
        else
            tx_cnt <= tx_cnt + 1;

    assign MISO = SS      ? 1'bz :
                  `SHIFT_DIRECTION ? miso_shift_reg[tx_cnt] :                   //LSB ->MSB
                                     miso_shift_reg[`DATA_LENGTH-tx_cnt-1] ;    //MSB -> LSB
    
end
else begin
    always@(negedge SCLK )
        if(SS)
            tx_cnt <= 0;
        else if(tx_cnt >= `DATA_LENGTH - 1)begin
            miso_shift_reg <= mosi_shift_reg;
            tx_cnt <= 0;
        end
        else 
            tx_cnt <= tx_cnt + 1;

    assign MISO = SS      ? 1'bz :
                  `SHIFT_DIRECTION ? miso_shift_reg[tx_cnt] :                   //LSB ->MSB
                                     miso_shift_reg[`DATA_LENGTH-tx_cnt-1] ;    //MSB -> LSB
 
end


endmodule

